using System;
using System.Collections.Generic;
using System.Text;
using FarsiLibrary.Utils;
using System.Windows.Forms;
namespace RtMonyTextBox
{
    public class RtMonyTextBox:TextBox
    {
        string[] masks = new string[] { "#", "##", "###", "#,###", "##,###", "###,###", "#,###,###", "##,###,###", "###,###,###", "#,###,###,###" };
        private bool thousandsseparators=true;
        private bool floatnumber;

        public bool ThousandsSeparators
        {
            get
            {
                return thousandsseparators; 
            }
            set 
            { 
                thousandsseparators = value; 
            }
        }

        public override string Text
        {
            get
            {
                return toEnglish.Convert(base.Text);
            }
            set
            {
                    base.Text = toFarsi.Convert(value);
            }
        }

        protected override void OnKeyUp(KeyEventArgs e)
        {
            base.OnKeyUp(e);
        }
        protected override void OnEnter(EventArgs e)
        {
            if (Value == 0)
                this.Text = "";
            base.OnEnter(e);
        }

        protected override void OnLeave(EventArgs e)
        {
            if (this.Text == "")
                this.Text = "0";
            base.OnLeave(e);
        }

        string _currentTypedValue = "";

         protected override void OnKeyDown(KeyEventArgs e)
         {
             // get the character that was pressed
             char keycode = (char)e.KeyValue;

             // determine if the character pressed was numeric or
             // if the character would go beyond the limit of the largest mask
             if ((!IsNumeric(keycode)) || Text.Length >= masks[masks.Length - 1].Length)
             {
                 // if it's not numeric, don't try to handle it
                 // and suppress the character so it won't be displayed
                 e.Handled = true;
                 e.SuppressKeyPress = true;
                 if (keycode != '\b')
                 {
                     // if it's a backspace, we still want to react to the keypress
                     return;
                 }
             }

             // add the character to the final string if its not a backspace
             if (keycode != '\b')
             {
                 if(Text.Length<this.MaxLength)
                     Text += toFarsi.Convert(KeycodeToDigit(keycode));
             }

             // take out all the commas, to get the raw number
             _currentTypedValue = SubtractCommas(Text);

             // handle the backspace by removing the digit at the end of the string
             if (keycode == '\b') //  take out a character
             {
                 if (base.SelectedText.Length > 0)
                 {
                     _currentTypedValue = _currentTypedValue.Replace(SelectedText, string.Empty);
                 }
                 else if (SelectionStart >= 0)
                 {
                     if (_currentTypedValue.Length > 0)
                     {
                         _currentTypedValue = _currentTypedValue.Remove(_currentTypedValue.Length - 1, 1);
                     }
                 }

             }

             // Put the commas back in the number string using the
             // mask that matches the number string length
             if(this.thousandsseparators)
                 _currentTypedValue = AddCommas(_currentTypedValue);

             // Assign the Text property of the edit box to the
             // comma decorated string
             Text = _currentTypedValue;
            /* if (Text.Length > 0)
             {
                 // select the last character
                 Select(Text.Length - 1, 1);
             }*/
             e.Handled = true;
             e.SuppressKeyPress = true;
         }

        private string KeycodeToDigit(char keycode)
        {
            int val = (int)keycode;
            if (char.IsDigit(keycode))
                return keycode.ToString();
            else if ((95 <val)&&(val< 106))
            {
                switch (val)
                {
                    case 96:
                        return "0";
                    case 97:
                        return "1";
                    case 98:
                        return "2";
                    case 99:
                        return "3";
                    case 100:
                        return "4";
                    case 101:
                        return "5";
                    case 102:
                        return "6";
                    case 103:
                        return "7";
                    case 104:
                        return "8";
                    case 105:
                        return "9";
                }
            }
            return keycode.ToString();
        }
        private bool IsNumeric(char val)
        {
            return char.IsDigit(val) || ((95 < (int)val)&&((int)val < 106));
        }

        private string AddCommas(string text)
        {
            string mask = "";
            if (text.Length > 0)
            {
                mask = masks[text.Length - 1];
            }

            for (int i = 0; i < text.Length; i++)
            {
                int index = mask.IndexOf('#');
                if (index >= 0)
                {
                    mask = mask.Insert(index, text[i].ToString());
                    mask = mask.Remove(index + 1, 1);
                }
            }

            return mask;
        }

        private string SubtractCommas(string text)
        {
            while (text.Contains(","))
            {
                text = text.Remove(text.IndexOf(','), 1);
            }

            return text;
        }

        public int Value
        {
            get
            {
                if (Text == "")
                    return 0;
                else
                    return int.Parse(SubtractCommas(toEnglish.Convert(Text)));
            }
            set 
            {
                if (value == 0)
                    Text =toFarsi.Convert("0");
                else
                    PlaceValueInTextBox(value);
            }
        }

        private void PlaceValueInTextBox(int _value)
        {
            string val = Convert.ToString(_value);
            if (_value > 0)
            {
                if (thousandsseparators)
                    Text = AddCommas(val);
                else
                    Text = _value.ToString();
                Select(0, Text.Length);
            }
        }
        public bool FloatNumber
        {
            get 
            {
                return floatnumber; 
            }
            set
            {
                floatnumber = value; 
            }
        }
    }
}